// main [|@"C:\Users\Hans\lib\webCat0.txt"; @"C:\Users\Hans\lib\webCat1.txt" |] ;;
// main [|@"C:\Users\Hans\lib\webCat1.txt"; @"C:\Users\Hans\lib\webCat2.txt" |] ;;

//#r @"C:\Users\Hans\lib\TextProcessing.dll" ;;

open System     ;;
open System.IO  ;;
open System.Net ;;
open System.Collections.Generic ;;
open System.Text.RegularExpressions ;;
open System.Web ;;
open System.Xml ;;
open TextProcessing ;;

type infoType = StartInfo of int | EndDiv of int
                | RefInfo of string * string | EndOfFile;;
 
let rec nextInfo(r:XmlReader) =
  match r.Read() with
  | false -> EndOfFile
  | _     ->
  match r.NodeType with
  | XmlNodeType.Element ->
    match r.Name with
    | "div" when (r.GetAttribute "class" = 
                     "toclevel2 children")
          -> StartInfo (r.Depth)
    | "a" -> let path = r.GetAttribute "href"
             ignore(r.Read())
             RefInfo(r.Value,path)
    |  _  -> nextInfo r 
  | XmlNodeType.EndElement when r.Name = "div" 
          -> EndDiv (r.Depth)
  |  _    -> nextInfo r ;;

let rec anyRefs(r:XmlReader) =
  match nextInfo r with
  | StartInfo n -> Some n
  | EndOfFile   -> None
  | _           -> anyRefs r ;;

let regQuote = Regex @"\042"  ;;
let quoteReplace str = regQuote.Replace(str,"'") ;; 
let cStr s = quoteReplace(HttpUtility.HtmlDecode s) ;;

let getWEBrefs(uri: string) = 
  let baseUri = Uri uri
  let webCl = new WebClient()
  let doc = webCl.DownloadString baseUri
  use docRd  = new StringReader(doc)
  let settings = 
      XmlReaderSettings(DtdProcessing = DtdProcessing.Ignore)
  use reader = XmlReader.Create(docRd,settings)
  let rec getRefs(n) =
    match nextInfo reader with
    | RefInfo(t,path) ->
      let pathUri = Uri(baseUri,path)   
      (cStr t, pathUri.AbsoluteUri) :: getRefs(n) 
    | EndDiv m       ->  
      if m <= n then [] else getRefs n
    | p              -> failwith ("getRefs error: " + (string p))  
  match anyRefs reader with
  | None   -> []
  | Some n -> getRefs n 

open System ;;
open System.IO ;;

let outputRef (output:StreamWriter) (title:string, uri:string) =
  output.WriteLine title 
  output.WriteLine uri;;

let expandUri (output:StreamWriter) uri =
  let lst = getWEBrefs uri
  List.iter (outputRef output) lst  ;;

let handleLinePair (output:StreamWriter) (rdr: StreamReader) = 
  ignore(rdr.ReadLine()) 
  expandUri output (rdr.ReadLine()) ;;

[<EntryPoint>]
let main (args: string[]) =
  if Array.length args < 2 then
    failwith "Missing parameters"
  else 
  if File.Exists args.[1] then
    failwith "Existing output file"
  else
    use output = File.CreateText args.[1]
    fileXiter (handleLinePair output) args.[0]
    output.Close() 
    0 ;;

